package testmodel.diagramA.part;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.OperationHistoryFactory;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.ui.URIEditorInput;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.emf.workspace.AbstractEMFOperation;
import org.eclipse.gef.commands.Command;
import org.eclipse.gmf.runtime.diagram.core.preferences.PreferencesHint;
import org.eclipse.gmf.runtime.diagram.core.services.ViewService;
import org.eclipse.gmf.runtime.diagram.ui.editparts.DiagramEditPart;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.HintedDiagramLinkStyle;
import org.eclipse.gmf.runtime.notation.NotationPackage;
import org.eclipse.gmf.runtime.notation.Style;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.HandlerUtil;

public class OpenTypeBAction extends AbstractHandler
	{
	TestModelDiagramEditor testModelDiagramEditor = null;
	private HintedDiagramLinkStyle diagramFacet = null;
	private String diagramKind = "TestModelB"; //$NON-NLS-1$
	private String editorID = "testmodel.diagramB.part.TestModelDiagramEditorID"; //$NON-NLS-1$
	
	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException 
		{
		testModelDiagramEditor = (TestModelDiagramEditor)HandlerUtil.getActiveEditorChecked(event);
		DiagramEditPart diagramEditPart = testModelDiagramEditor.getDiagramEditPart();
		
		if (diagramEditPart.getModel() instanceof View) 
			{
			View view = (View)diagramEditPart.getModel();
			Style link = view.getStyle(NotationPackage.eINSTANCE.getHintedDiagramLinkStyle());
				
			if (link instanceof HintedDiagramLinkStyle) 
				{
				diagramFacet = (HintedDiagramLinkStyle)link;
				
				Command setHintCommand = new Command()
					{
					@Override
					public void execute()
						{
						AbstractEMFOperation doSetHintOp = new AbstractEMFOperation(testModelDiagramEditor.getEditingDomain(),"Set Diagram Hint to TestModelB")
							{
							@Override
							protected IStatus doExecute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException 
								{
								diagramFacet.setHint(diagramKind);
								return Status.OK_STATUS;
								}	
							};
							
						try 
							{OperationHistoryFactory.getOperationHistory().execute(doSetHintOp, null, null);} 
						catch (ExecutionException e) 
							{}
						
						}
					};
				
				testModelDiagramEditor.getDiagramEditDomain().getDiagramCommandStack().execute(setHintCommand);
				
				try 
					{
					Diagram diagram = getDiagramToOpen();
					
					if (diagram == null) 
						{diagram = intializeNewDiagram();}
					
					URI uri = EcoreUtil.getURI(diagram);
					String editorName = uri.lastSegment() + '#' + diagram.eResource().getContents().indexOf(diagram);
					IEditorInput editorInput = new URIEditorInput(uri, editorName);
					IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
					page.openEditor(editorInput, editorID);
					}
				catch (Exception ex) 
					{System.err.println("Can't open diagram: " + ex.getMessage());}	
				
				Command clearHintCommand = new Command()
					{
					@Override
					public void execute()
						{
						AbstractEMFOperation doClearHintOp = new AbstractEMFOperation(testModelDiagramEditor.getEditingDomain(),"Set Diagram Hint to TestModelB")
							{
							@Override
							protected IStatus doExecute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException 
								{
								diagramFacet.setHint(null);
								return Status.OK_STATUS;
								}	
							};
							
						try 
							{OperationHistoryFactory.getOperationHistory().execute(doClearHintOp, null, null);} 
						catch (ExecutionException e) 
							{}
						
						}
					};
				
				testModelDiagramEditor.getDiagramEditDomain().getDiagramCommandStack().execute(clearHintCommand);
			
				}
			}
			
		return null;
		}
	
	protected Diagram getDiagramToOpen() 
		{return diagramFacet.getDiagramLink();}

/**
 * @generated 
 */
protected Diagram intializeNewDiagram() throws ExecutionException
	{
	EObject diagramDomainElement = getDiagramDomainElement();

	// Create the diagram using the element that was updated
	final Diagram d = ViewService.createDiagram(diagramDomainElement, diagramKind, getPreferencesHint());
	
	if (d == null) 
		{throw new ExecutionException("Can't create diagram of '" + diagramKind + "' kind");}
	
	
	Command setLinkAndResourceCommand = new Command()
		{
		public void execute()
			{
			AbstractEMFOperation doSetLinkAndResourceOp = new AbstractEMFOperation(testModelDiagramEditor.getEditingDomain(),"Set Diagram Hint to TestModelB")
				{
				@Override
				protected IStatus doExecute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException 
					{
					diagramFacet.setDiagramLink(d);
					assert diagramFacet.eResource() != null;
					diagramFacet.eResource().getContents().add(d);
					return Status.OK_STATUS;
					}	
				};
				
			try 
				{OperationHistoryFactory.getOperationHistory().execute(doSetLinkAndResourceOp, null, null);} 
			catch (ExecutionException e) 
				{}
			}
		};

	
	testModelDiagramEditor.getDiagramEditDomain().getDiagramCommandStack().execute(setLinkAndResourceCommand);
	
	return d;
	}

protected EObject getDiagramDomainElement() 
	{
	// use same element as associated with EP
	return ((View) diagramFacet.eContainer()).getElement();
	}

protected PreferencesHint getPreferencesHint() 
	{
	// XXX prefhint from target diagram's editor?
	return TestModelDiagramEditorPlugin.DIAGRAM_PREFERENCES_HINT;
	}

	}
