package testmodel.diagramA.part;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.emf.common.ui.URIEditorInput;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.diagram.core.preferences.PreferencesHint;
import org.eclipse.gmf.runtime.diagram.core.services.ViewService;
import org.eclipse.gmf.runtime.notation.Diagram;
import org.eclipse.gmf.runtime.notation.HintedDiagramLinkStyle;
import org.eclipse.gmf.runtime.notation.NotationPackage;
import org.eclipse.gmf.runtime.notation.Style;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

public class TestModelOpenTypeBEditorAction implements IEditorActionDelegate 
	{
	private IEditorPart targetPart;
	private HintedDiagramLinkStyle diagramFacet;
	private String diagramKind = "TestModelB"; //$NON-NLS-1$
	private String editorID = "testmodel.diagramB.part.TestModelDiagramEditorID"; //$NON-NLS-1$
	
	
	@Override
	public void setActiveEditor(IAction action, IEditorPart targetEditor) {
		targetPart = targetEditor;	
	}

	
	@Override
	public void run(IAction action) 
		{
		EditPart targetEditPart = null;
		
		if (targetPart instanceof EditPart)
			{
			targetEditPart = (EditPart)targetPart;
		
			if (targetEditPart.getModel() instanceof View) 
				{
				View view = (View) targetEditPart.getModel();
				Style link = view.getStyle(NotationPackage.eINSTANCE.getHintedDiagramLinkStyle());
					
				if (link instanceof HintedDiagramLinkStyle) 
					{
					diagramFacet = (HintedDiagramLinkStyle)link;
					
					diagramFacet.setHint(diagramKind);
			
					try 
						{
						Diagram diagram = getDiagramToOpen();
						
						if (diagram == null) 
							{diagram = intializeNewDiagram();}
						
						URI uri = EcoreUtil.getURI(diagram);
						String editorName = uri.lastSegment() + '#' + diagram.eResource().getContents().indexOf(diagram);
						IEditorInput editorInput = new URIEditorInput(uri, editorName);
						IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
						page.openEditor(editorInput, editorID);
						//return CommandResult.newOKCommandResult();
						}
					catch (Exception ex) 
						{System.err.println("Can't open diagram: " + ex.getMessage());}	
					}
				}
			}
		}
	

	@Override
	public void selectionChanged(IAction action, ISelection selection) {
		// TODO Auto-generated method stub
		
	}
	
	protected Diagram getDiagramToOpen() 
		{return diagramFacet.getDiagramLink();}

	/**
	 * @generated 
	 */
	protected Diagram intializeNewDiagram() throws ExecutionException
		{
		// MOD Update added to ensure the required elements are added to the diagram
		EObject diagramDomainElement = getDiagramDomainElement();

		// Create the diagram using the element that was updated
		Diagram d = ViewService.createDiagram(diagramDomainElement, diagramKind, getPreferencesHint());
		
		// endMOD
		if (d == null) 
			{throw new ExecutionException("Can't create diagram of '" + diagramKind + "' kind");}
		
		diagramFacet.setDiagramLink(d);
		assert diagramFacet.eResource() != null;
		diagramFacet.eResource().getContents().add(d);
		return d;
		}
	
	protected EObject getDiagramDomainElement() 
		{
		// use same element as associated with EP
		return ((View) diagramFacet.eContainer()).getElement();
		}
	
	protected PreferencesHint getPreferencesHint() 
		{
		// XXX prefhint from target diagram's editor?
		return TestModelDiagramEditorPlugin.DIAGRAM_PREFERENCES_HINT;
		}
	
	}
